from time import sleep

from .sendmail import sendmail
from .sendsms import sendsms
from .itemize import itemize


class Notifier:

    def __init__(self, title="slic notification", wait_time=0.1):
        self.title = title
        self.wait_time = wait_time

        self.funcs  = set()
        self.mails  = set()
        self.phones = set()


    def add_many(self, recipients):
        """
        the recipients sequence may contain email addresses, phone numbers and functions
        """
        for i in recipients:
            self.add(i)


    def add(self, recipient):
        """
        recipient may be an email address, a phone number or a function
        """
        if callable(recipient):
            self.funcs.add(recipient)
        elif "@" in str(recipient):
            self.mails.add(recipient)
        else:
            self.phones.add(recipient)


    def clear(self):
        self.funcs.clear()
        self.mails.clear()
        self.phones.clear()


    def notify(self, message, title=None):
        title = title or self.title

        for f in self.funcs:
            f(message)
            sleep(self.wait_time)

        for m in self.mails:
            sendmail(m, subject=title, body=message)
            sleep(self.wait_time)

        for p in self.phones:
            sendsms(p, message)
            sleep(self.wait_time)


    def __repr__(self):
        func_names = (f.__name__ for f in self.funcs)
        printable_funcs  = itemize(func_names,  header="Functions")
        printable_mails  = itemize(self.mails,  header="Mail Adresses")
        printable_phones = itemize(self.phones, header="Phone Numbers")

        res = (printable_funcs, printable_mails, printable_phones)
        return "\n\n".join(res)



