import ast
import epics

from .cfgfile import parse_file, parse_string


COMPARISONS = sorted(epics.Alarm.ops)
PRINTABLE_COMPARISONS = ", ".join(COMPARISONS)


def read_rules_file(fn):
    for line in parse_file(fn):
        yield parse_line(line)

def read_rules_string(string):
    for line in parse_string(string):
        yield parse_line(line)


def parse_line(line):
    pieces = line.split(maxsplit=2)
    pieces = [i.strip() for i in pieces]
    pvname, comparison, trip_point = pieces
    check_comparison(comparison)
    trip_point = parse_trip_point(trip_point)
    return pvname, comparison, trip_point


def check_comparison(comparison):
    if comparison not in COMPARISONS:
        raise ValueError(f'comparison {comparison} is not known: {PRINTABLE_COMPARISONS}')


def parse_trip_point(trip_point):
    #TODO: enforce quotation marks or not?
    try:
        return ast.literal_eval(trip_point)
    except (ValueError, SyntaxError):
#        raise ValueError(f'trip point {trip_point} cannot be parsed') from e
        return trip_point # use the original value as string



